<?php

require get_template_directory() . '/inc/constants.php';


function phone_repair_content_width() {
	$GLOBALS['content_width'] = apply_filters( 'phone_repair_content_width', 1170 );
}
add_action( 'after_setup_theme', 'phone_repair_content_width', 0 );

function phone_repair_setup() {
	// Theme translation support.
	load_theme_textdomain( 'phone-repair', get_template_directory() . '/languages' );
	// Image sizes.
	add_image_size( 'phone-repair-360x185-cropped', 360, 185, true );
	add_image_size( 'phone-repair-360x360-cropped', 360, 360, true );
	// Theme support.
	add_theme_support( 'automatic-feed-links' );
	add_theme_support( 'title-tag' );
	add_theme_support( 'post-thumbnails' );
	add_theme_support( 'custom-header', array(
		'uploads'     => true,
		'flex-width'  => true,
		'width'       => 2000,
		'flex-height' => true,
		'height'      => 600,
	) );
	add_theme_support( 'custom-background', array(
		'default-color' => 'ffffff',
	) );
	add_theme_support( 'html5', array(
		'search-form',
		'comment-form',
		'comment-list',
		'gallery',
		'caption',
	) );
	add_theme_support( 'post-formats', array(
		'aside',
		'image',
		'video',
		'quote',
		'link',
		'gallery',
		'status',
	) );
	// Add theme support for selective refresh for widgets.
	add_theme_support( 'customize-selective-refresh-widgets' );
	// WooCommerce support.
	add_theme_support( 'woocommerce' );
	add_theme_support( 'wc-product-gallery-lightbox' );
	add_theme_support( 'wc-product-gallery-slider' );
	// Navigation menus.
	register_nav_menus( array(
		'primary' => esc_html__( 'Primary', 'phone-repair' ),
	) );
}
add_action( 'after_setup_theme', 'phone_repair_setup' );

function phone_repair_widgets_init() {
	register_sidebar( array(
		'name' => __( 'Blog Sidebar', 'phone-repair' ),
		'id' => 'blog-sidebar',
		'before_widget' => '<aside id="%1$s" class="widget widget--sidebar %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h3 class="widget__title">',
		'after_title' => '</h3>',
	) );
	register_sidebar( array(
		'name' => __( 'Shop Sidebar', 'phone-repair' ),
		'id' => 'shop-sidebar',
		'before_widget' => '<aside id="%1$s" class="widget widget--sidebar %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h3 class="widget__title">',
		'after_title' => '</h3>',
	) );
	register_sidebar( array(
		'name' => __( 'VC Sidebar', 'phone-repair' ),
		'id' => 'vc-sidebar',
		'before_widget' => '<aside id="%1$s" class="widget widget--sidebar %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h3 class="widget__title">',
		'after_title' => '</h3>',
	) );
	register_sidebar( array(
		'name' => __( 'Footer Widget Area 1', 'phone-repair' ),
		'id' => 'footer-widgets-1',
		'before_widget' => '<aside id="%1$s" class="widget widget--footer %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h3 class="widget__title">',
		'after_title' => '</h3>',
	) );
	register_sidebar( array(
		'name' => __( 'Footer Widget Area 2', 'phone-repair' ),
		'id' => 'footer-widgets-2',
		'before_widget' => '<aside id="%1$s" class="widget widget--footer %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h3 class="widget__title">',
		'after_title' => '</h3>',
	) );
	register_sidebar( array(
		'name' => __( 'Footer Widget Area 3', 'phone-repair' ),
		'id' => 'footer-widgets-3',
		'before_widget' => '<aside id="%1$s" class="widget widget--footer %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h3 class="widget__title">',
		'after_title' => '</h3>',
	) );
}
add_action( 'widgets_init', 'phone_repair_widgets_init' );

function phone_repair_enqueue_scripts() {
	// Enqueue theme styles
	wp_enqueue_style( 'phone-repair-google-fonts', phone_repair_google_fonts_url() );
	wp_enqueue_style( 'phone-repair-libs', get_template_directory_uri() . '/assets/css/theme-libs.min.css', array(), PHONE_REPAIR_THEME_VERSION );
	wp_enqueue_style( 'phone-repair-style', get_template_directory_uri() . '/assets/css/theme.css', array(), PHONE_REPAIR_THEME_VERSION );
	if ( is_rtl() ) {
		wp_enqueue_style( 'phone-repair-rtl', get_template_directory_uri() . '/assets/css/rtl.css', array(), PHONE_REPAIR_THEME_VERSION );
	}
	// Enqueue theme scripts
	wp_enqueue_script( 'phone-repair-libs', get_template_directory_uri() . '/assets/js/theme-libs.min.js', array( 'jquery' ), PHONE_REPAIR_THEME_VERSION, true );
	wp_enqueue_script( 'phone-repair-script', get_template_directory_uri() . '/assets/js/theme.js', array( 'jquery', 'jquery-effects-core' ), PHONE_REPAIR_THEME_VERSION, true );
	// Comments
	if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
		wp_enqueue_script( 'comment-reply' );
	}
	// Inline styles
	$inline_css = phone_repair_get_inline_styles();
	if ( ! empty( $inline_css ) ) {
		wp_add_inline_style( 'phone-repair-style', $inline_css );
	}
	// Localization
	$phone_repair_l10n = array(
		'expand'   => __( 'Expand submenu', 'phone-repair' ),
		'collapse' => __( 'Collapse submenu', 'phone-repair' ),
	);
	wp_localize_script( 'phone-repair-script', 'phonerepairScreenReaderText', $phone_repair_l10n );
}
add_action( 'wp_enqueue_scripts', 'phone_repair_enqueue_scripts' );

function phone_repair_wp_kses_allowed_html( $tags, $context ) {
	if ( 'breadcrumb_navxt' === $context ) {
		global $breadcrumb_navxt;
		return $breadcrumb_navxt->allowed_html( array() );
	}

	if ( 'post' !== $context && 'phone_repair_top_bar' !== $context ) {
		return $tags;
	}

	if ( 'phone_repair_top_bar' === $context ) {
		global $allowedposttags;
		$tags = $allowedposttags;
		$tags['a']['onclick'] = true;
		$tags['button']['onclick'] = true;
	}

	$tags['a']['data-text'] = true;
	$tags['span']['class'] = true;

	if ( ! isset( $tags['time'] ) ) {
		$tags['time'] = array();
	}
	$tags['time']['class'] = true;
	$tags['time']['datetime'] = true;

	return $tags;
}
add_filter( 'wp_kses_allowed_html', 'phone_repair_wp_kses_allowed_html', 10, 2 );

function phone_repair_the_content_more_link( $element, $text ) {
	return '<a class="more-link" href="' . esc_url( get_permalink() . '#more-' . get_the_ID() ) . '" data-text="' . esc_attr( $text ) . '">' . esc_html( $text ) . '</a>';
}
add_filter( 'the_content_more_link', 'phone_repair_the_content_more_link', 10, 2 );

function phone_repair_tag_cloud_args( $args ) {
	$args['smallest'] = $args['largest'] = 1;
	$args['unit'] = 'em';
	return $args;
}
add_filter( 'widget_tag_cloud_args', 'phone_repair_tag_cloud_args' );

function phone_repair_vc_before_init() {
	vc_set_as_theme();
	if ( function_exists( 'vc_set_default_editor_post_types' ) ) {
		vc_set_default_editor_post_types( array( 'page', 'service' ) );
	}
}
add_action( 'vc_before_init', 'phone_repair_vc_before_init' );

function phone_repair_post_teaser_templates( $templates ) {
	return array(
		esc_html__( 'Post Teaser 1', 'phone-repair' ) => 'partials/templates/post-teaser',
		esc_html__( 'Post Teaser 2', 'phone-repair' ) => 'partials/templates/post-teaser2',
		esc_html__( 'Post Teaser 3', 'phone-repair' ) => 'partials/templates/post-teaser3',
		esc_html__( 'Post Teaser 4', 'phone-repair' ) => 'partials/templates/post-teaser4',
	);
}
add_filter( 'brtm_post_teaser_templates', 'phone_repair_post_teaser_templates' );
add_filter( 'brtm_post_grid_item_templates', 'phone_repair_post_teaser_templates' );
add_filter( 'brtm_post_carousel_item_templates', 'phone_repair_post_teaser_templates' );

function phone_repair_info_box_templates( $templates ) {
	return array(
		esc_html__( 'Info Box', 'phone-repair' ) => 'partials/templates/info-box',
	);
}
add_filter( 'brtm_info_box_templates', 'phone_repair_info_box_templates' );

add_filter( 'brtm_enqueue_themify_icons_css', '__return_false' );


require get_template_directory() . '/inc/template-tags.php';
require get_template_directory() . '/inc/extras.php';
require get_template_directory() . '/inc/custom-colors.php';
require get_template_directory() . '/inc/inline-styles.php';
require get_template_directory() . '/inc/customizer.php';
require get_template_directory() . '/inc/bem-menu-walker.php';
require get_template_directory() . '/inc/tgmpa/tgm-plugin-activation.php';
require get_template_directory() . '/inc/carbon-fields/carbon-fields.php';
require get_template_directory() . '/inc/easy-google-fonts/easy-google-fonts.php';
require get_template_directory() . '/inc/one-click-demo-import/one-click-demo-import.php';
require get_template_directory() . '/inc/woocommerce/woocommerce-functions.php';
require get_template_directory() . '/inc/woocommerce/woocommerce-hooks.php';
