<?php
/**
 * Integration with Easy Google Fonts plugin for some nice typography and styling
 * options via the Customizer.
 *
 * @package Phone_Repair
 */


/**
 * Change the priority of Typography section in the Customizer so that it appears
 * below Site Identity section.
 */
function phone_repair_tt_font_get_panels( $panels ) {
	if ( isset( $panels['tt_font_typography_panel'] ) ) {
		$panels['tt_font_typography_panel']['priority'] = 30;
	}
	return $panels;
}
add_filter( 'tt_font_get_panels', 'phone_repair_tt_font_get_panels' );


/**
 * Add additional typography options to allow customization of some theme elements.
 */
function phone_repair_tt_font_get_option_parameters( $options ) {
	$theme_options = array(
		'phone_repair_tt_body' => array(
			'name'       => 'phone_repair_tt_body',
			'title'      => __( 'Body', 'phone-repair' ),
			'properties' => array( 'selector' => apply_filters( 'phone_repair_tt_body', 'body' ) ),
		),
		'phone_repair_tt_headings' => array(
			'name'       => 'phone_repair_tt_headings',
			'title'      => __( 'Headings', 'phone-repair' ),
			'properties' => array( 'selector' => apply_filters( 'phone_repair_tt_headings', 'h1, h2, h3, h4, h5, h6, .h1, .h2, .h3, .h4, .h5, .h6' ) ),
		),
		'phone_repair_tt_primary_menu' => array(
			'name'       => 'phone_repair_tt_primary_menu',
			'title'      => __( 'Menu', 'phone-repair' ),
			'properties' => array( 'selector' => apply_filters( 'phone_repair_tt_primary_menu', '.primary-menu__item > a, .primary-menu__sub-menu-item > a' ) ),
		),
		'phone_repair_tt_page_title' => array(
			'name'       => 'phone_repair_tt_page_title',
			'title'      => __( 'Page Title', 'phone-repair' ),
			'properties' => array( 'selector' => apply_filters( 'phone_repair_tt_page_title', '.page-title__title' ) ),
		),
		'phone_repair_tt_breadcrumbs' => array(
			'name'       => 'phone_repair_tt_breadcrumbs',
			'title'      => __( 'Breadcrumbs', 'phone-repair' ),
			'properties' => array( 'selector' => apply_filters( 'phone_repair_tt_breadcrumbs', '.page-title__breadcrumbs' ) ),
		),
		'phone_repair_tt_link' => array(
			'name'       => 'phone_repair_tt_link',
			'title'      => __( 'Link', 'phone-repair' ),
			'properties' => array( 'selector' => apply_filters( 'phone_repair_tt_link', 'a, .more-link, .more-link:hover, .more-link:focus' ) ),
		),
		'phone_repair_tt_link_hover' => array(
			'name'       => 'phone_repair_tt_link_hover',
			'title'      => __( 'Link hover', 'phone-repair' ),
			'properties' => array( 'selector' => apply_filters( 'phone_repair_tt_link_hover', 'a:hover, a:focus, .more-link::before' ) ),
		),
		'phone_repair_tt_footer_text' => array(
			'name'       => 'phone_repair_tt_footer_text',
			'title'      => __( 'Footer text', 'phone-repair' ),
			'properties' => array( 'selector' => apply_filters( 'phone_repair_tt_footer_text', '.widget--footer' ) ),
		),
		'phone_repair_tt_footer_heading' => array(
			'name'       => 'phone_repair_tt_footer_heading',
			'title'      => __( 'Footer headings', 'phone-repair' ),
			'properties' => array(
				'selector' => apply_filters( 'phone_repair_tt_footer_heading',
					'.widget--footer h1, .widget--footer h2, .widget--footer h3, .widget--footer h4, .widget--footer h5, .widget--footer h6,
					.widget--footer .h1, .widget--footer .h2, .widget--footer .h3, .widget--footer .h4, .widget--footer .h5, .widget--footer .h6,
					.widget--footer .widget__title'
				),
			),
		),
		'phone_repair_tt_footer_link' => array(
			'name'       => 'phone_repair_tt_footer_link',
			'title'      => __( 'Footer Link', 'phone-repair' ),
			'properties' => array( 'selector' => apply_filters( 'phone_repair_tt_footer_link', '.widget--footer a' ) ),
		),
		'phone_repair_tt_footer_link_hover' => array(
			'name'       => 'phone_repair_tt_footer_link_hover',
			'title'      => __( 'Footer Link hover', 'phone-repair' ),
			'properties' => array( 'selector' => apply_filters( 'phone_repair_tt_footer_link_hover', '.widget--footer a:hover' ) ),
		),
		'phone_repair_tt_button' => array(
			'name'       => 'phone_repair_tt_button',
			'title'      => __( 'Button', 'phone-repair' ),
			'properties' => array(
				'selector' => apply_filters( 'phone_repair_tt_button',
					'a.button,
					button.button,
					input.button,
					.woocommerce a.button.alt,
					.woocommerce button.button.alt,
					.woocommerce input.button.alt,
					.woocommerce #respond input#submit.alt,
					.woocommerce a.added_to_cart.alt,
					.woocommerce a.button.alt.disabled,
					.woocommerce a.button.alt:disabled,
					.woocommerce a.button.alt:disabled[disabled],
					.woocommerce a.button.alt.disabled:hover,
					.woocommerce a.button.alt:disabled:hover,
					.woocommerce a.button.alt:disabled[disabled]:hover,
					.woocommerce button.button.alt.disabled,
					.woocommerce button.button.alt:disabled,
					.woocommerce button.button.alt:disabled[disabled],
					.woocommerce button.button.alt.disabled:hover,
					.woocommerce button.button.alt:disabled:hover,
					.woocommerce button.button.alt:disabled[disabled]:hover,
					.woocommerce input.button.alt.disabled,
					.woocommerce input.button.alt:disabled,
					.woocommerce input.button.alt:disabled[disabled],
					.woocommerce input.button.alt.disabled:hover,
					.woocommerce input.button.alt:disabled:hover,
					.woocommerce input.button.alt:disabled[disabled]:hover,
					.woocommerce #respond input#submit.alt.disabled,
					.woocommerce #respond input#submit.alt:disabled,
					.woocommerce #respond input#submit.alt:disabled[disabled],
					.woocommerce #respond input#submit.alt.disabled:hover,
					.woocommerce #respond input#submit.alt:disabled:hover,
					.woocommerce #respond input#submit.alt:disabled[disabled]:hover,
					.woocommerce a.added_to_cart.alt.disabled,
					.woocommerce a.added_to_cart.alt:disabled,
					.woocommerce a.added_to_cart.alt:disabled[disabled],
					.woocommerce a.added_to_cart.alt.disabled:hover,
					.woocommerce a.added_to_cart.alt:disabled:hover,
					.woocommerce a.added_to_cart.alt:disabled[disabled]:hover'
				),
			),
		),
		'phone_repair_tt_button_hover' => array(
			'name'       => 'phone_repair_tt_button_hover',
			'title'      => __( 'Button hover', 'phone-repair' ),
			'properties' => array(
				'selector' => apply_filters( 'phone_repair_tt_button_hover',
					'a.button:hover, a.button:focus,
					button.button:hover, button.button:focus,
					input.button:hover, input.button:focus,
					.woocommerce a.button:hover,
					.woocommerce button.button:hover,
					.woocommerce input.button:hover,
					.woocommerce #respond input#submit:hover,
					.woocommerce a.added_to_cart:hover,
					.woocommerce a.button.alt:hover,
					.woocommerce button.button.alt:hover,
					.woocommerce input.button.alt:hover,
					.woocommerce #respond input#submit.alt:hover,
					.woocommerce a.added_to_cart.alt:hover'
				),
			),
		),
	);
	$options = array_merge( $theme_options, $options );
	return $options;
}
add_filter( 'tt_font_get_option_parameters', 'phone_repair_tt_font_get_option_parameters' );


/**
 * The format of the default CSS selector for h1-h6.
 */
function phone_repair_tt_heading_selector( $level ) {
	return sprintf( 'h%1$d, .h%1$d', $level );
}

/**
 * Override the default h1 selector.
 */
function phone_repair_tt_default_heading_h1() {
	return phone_repair_tt_heading_selector( 1 );
}
add_filter( 'tt_default_heading_1', 'phone_repair_tt_default_heading_h1' );

/**
 * Override the default h2 selector.
 */
function phone_repair_tt_default_heading_h2() {
	return phone_repair_tt_heading_selector( 2 );
}
add_filter( 'tt_default_heading_h2', 'phone_repair_tt_default_heading_h2' );

/**
 * Override the default h3 selector.
 */
function phone_repair_tt_default_heading_h3() {
	return phone_repair_tt_heading_selector( 3 );
}
add_filter( 'tt_default_heading_h3', 'phone_repair_tt_default_heading_h3' );

/**
 * Override the default h4 selector.
 */
function phone_repair_tt_default_heading_h4() {
	return phone_repair_tt_heading_selector( 4 );
}
add_filter( 'tt_default_heading_h4', 'phone_repair_tt_default_heading_h4' );

/**
 * Override the default h5 selector.
 */
function phone_repair_tt_default_heading_h5() {
	return phone_repair_tt_heading_selector( 5 );
}
add_filter( 'tt_default_heading_h5', 'phone_repair_tt_default_heading_h5' );

/**
 * Override the default h6 selector.
 */
function phone_repair_tt_default_heading_h6() {
	return phone_repair_tt_heading_selector( 6 );
}
add_filter( 'tt_default_heading_h6', 'phone_repair_tt_default_heading_h6' );
