<?php
/**
 * One Click Demo Import plugin integration.
 *
 * @package PhoneRepair
 */

add_filter( 'pt-ocdi/import_files', 'phone_repair_demo_import_config' );
add_filter( 'pt-ocdi/timeout_for_downloading_import_file', 'phone_repair_timeout_for_downloading_import_file' );

add_action( 'wp_import_insert_term', 'phone_repair_save_imported_nav_menu_id', 10, 2 );
add_action( 'wp_import_insert_post', 'phone_repair_save_imported_post_id',     10, 2 );

add_action( 'pt-ocdi/after_import', 'phone_repair_replace_demo_urls' );
add_action( 'pt-ocdi/after_import', 'phone_repair_replace_nav_menu_shortcode_atts' );
add_action( 'pt-ocdi/after_import', 'phone_repair_delete_import_mappings' );

add_action( 'pt-ocdi/after_import', 'phone_repair_assign_static_home_page' );
add_action( 'pt-ocdi/after_import', 'phone_repair_assign_nav_menu_locations' );
add_action( 'pt-ocdi/after_import', 'phone_repair_update_revslider_responsive_settings' );
add_action( 'pt-ocdi/after_import', 'phone_repair_import_revslider_files' );

function phone_repair_demo_import_config() {
	$demo_host = 'https://dl.brothersthemes.com/import/phone-repair/';
	$upload_dir = wp_upload_dir();
	return array(
		array(
			'import_file_name'           => esc_html__( 'Phone Repair', 'phone-repair' ),
			'import_file_url'            => $demo_host . 'demo-content.xml',
			'import_customizer_file_url' => $demo_host . 'demo-customizer.dat',
			'import_widget_file_url'     => $demo_host . 'demo-widgets.wie',
			'import_revslider_file_urls' => array(
				'home-slider-1' => $demo_host . 'home-slider-1.zip',
				'home-slider-2' => $demo_host . 'home-slider-2.zip',
				'home-slider-3' => $demo_host . 'home-slider-3.zip',
			 ),
			'import_nav_menu_locations'  => array(
				'menu-1' => array( 'primary' ),
			),
			'import_replace_urls'        => array(
				'https://demo.brothersthemes.com/phone-repair/wp-content/uploads/sites/3/' => trailingslashit( $upload_dir['baseurl'] ),
				'https://demo.brothersthemes.com/phone-repair/'                            => home_url( '/' ),
			),
		),
	);
}

function phone_repair_timeout_for_downloading_import_file() {
	return 120; // seconds
}

function phone_repair_get_import_mappings() {
	$mappings = get_transient( 'phone_repair_import_mappings' );
	if ( ! is_array( $mappings ) ) {
		$mappings = array(
			'post_id'     => array(),
			'nav_menu_id' => array(),
		);
	}
	return $mappings;
}

function phone_repair_save_import_mappings( $mappings ) {
	set_transient( 'phone_repair_import_mappings', $mappings, 0.1 * HOUR_IN_SECONDS );
}

function phone_repair_delete_import_mappings() {
	delete_transient( 'phone_repair_import_mappings' );
}

function phone_repair_save_imported_nav_menu_id( $term_id, $data ) {
	if ( 'nav_menu' !== $data['taxonomy'] || ! isset( $data['id'] ) ) {
		return;
	}
	$original_id = $data['id'];
	$new_id = $term_id;
	$mappings = phone_repair_get_import_mappings();
	$mappings['nav_menu_id'][ $original_id ] = $new_id;
	phone_repair_save_import_mappings( $mappings );
}

function phone_repair_save_imported_post_id( $post_id, $original_id ) {
	$mappings = phone_repair_get_import_mappings();
	$mappings['post_id'][ $original_id ] = $post_id;
	phone_repair_save_import_mappings( $mappings );
}

function phone_repair_replace_demo_urls( $import_config ) {
	$search = array_keys( $import_config['import_replace_urls'] );
	$search = array_merge( $search, array_map( 'rawurlencode', $search ) );
	$replace = array_values( $import_config['import_replace_urls'] );
	$replace = array_merge( $replace, array_map( 'rawurlencode', $replace ) );

	// Replace in imported posts content
	$mappings = phone_repair_get_import_mappings();
	foreach ( $mappings['post_id'] as $post_id ) {
		$post = get_post( $post_id );
		$content = $post->post_content;
		$new_content = str_replace( $search, $replace, $content );
		if ( $new_content !== $content ) {
			wp_update_post( array( 'ID' => $post_id, 'post_content' => $new_content ) );
		}
	}

	// Replace in text widgets content
	$widget_text = get_option( 'widget_text' );
	$widget_text = phone_repair_recursive_replace( $search, $replace, $widget_text );
	update_option( 'widget_text', $widget_text );
}

function phone_repair_replace_nav_menu_shortcode_atts() {
	$mappings = phone_repair_get_import_mappings();
	if ( empty( $mappings['nav_menu_id'] ) || empty( $mappings['post_id'] ) ) {
		return;
	}

	$old_atts = array();
	$new_atts = array();
	foreach ( $mappings['nav_menu_id'] as $old_id => $new_id ) {
		$old_atts[] = " nav_menu=\"$old_id\"";
		$new_atts[] = " nav_menu=\"$new_id\"";
	}

	foreach ( $mappings['post_id'] as $post_id ) {
		$post = get_post( $post_id );
		$content = $post->post_content;
		$new_content = str_replace( $old_atts, $new_atts, $content );
		if ( $new_content !== $content ) {
			wp_update_post( array( 'ID' => $post_id, 'post_content' => $new_content ) );
		}
	}
}

function phone_repair_assign_static_home_page() {
	$front_page = get_page_by_title( 'Home 1' );
	$blog_page  = get_page_by_title( 'Our news' );
	update_option( 'show_on_front', 'page' );
	update_option( 'page_on_front', $front_page->ID );
	update_option( 'page_for_posts', $blog_page->ID );
}

function phone_repair_assign_nav_menu_locations( $import_config ) {
	if ( ! isset( $import_config['import_nav_menu_locations'] ) ) {
		return;
	}

	$menu_locations = array();
	foreach ( $import_config['import_nav_menu_locations'] as $slug => $locations ) {
		if ( $menu_term = get_term_by( 'slug', $slug, 'nav_menu' ) ) {
			foreach ( $locations as $location ) {
				$menu_locations[ $location ] = $menu_term->term_id;
			}
		}
	}
	set_theme_mod( 'nav_menu_locations', $menu_locations );
}

function phone_repair_update_revslider_responsive_settings() {
	// Check if Revolution Slider plugin is active
	if ( ! class_exists( 'RevSlider' ) || ! class_exists( 'RevSliderOperations' ) ) {
		return;
	}

	// Check that the required API is available
	$operations = new RevSliderOperations();
	if ( ! method_exists( $operations, 'getGeneralSettingsValues' ) || ! method_exists( $operations, 'updateGeneralSettings' ) ) {
		return;
	}

	$responsive_widths = array(
		'width'          => 1400,
		'width_notebook' => 1200,
		'width_tablet'   => 992,
		'width_mobile'   => 768,
	);

	$global_settings = $operations->getGeneralSettingsValues();
	if ( ! is_array( $global_settings ) ) {
		$global_settings = array();
	}
	$global_settings = array_merge( $global_settings, $responsive_widths );
	$operations->updateGeneralSettings( $global_settings );
}
add_action( 'init', 'phone_repair_update_revslider_responsive_settings' );

function phone_repair_import_revslider_files( $import_config ) {
	if ( ! isset( $import_config['import_revslider_file_urls'] ) ) {
		return;
	}

	foreach ( $import_config['import_revslider_file_urls'] as $alias => $url ) {
		phone_repair_import_revslider( $alias, $url );
	}
}

function phone_repair_import_revslider( $alias, $file_url ) {
	if ( empty( $file_url ) || ! class_exists( 'RevSlider' ) || RevSlider::isAliasExists( $alias ) ) {
		return;
	}

	if ( ! is_admin() && defined( 'RS_PLUGIN_PATH' ) ) {
		include_once RS_PLUGIN_PATH . 'admin/includes/template.class.php';
		include_once RS_PLUGIN_PATH . 'admin/includes/import.class.php';
	}

	if ( strpos( $file_url, 'http://' ) === false && strpos( $file_url, 'https://' ) === false ) {
		$revslider = new RevSlider;
		$revslider->importSliderFromPost( false, false, $file_url );
		return;
	}

	// Initialize the WP filesystem
	$wp_filesystem = phone_repair_init_wp_filesystem();
	if ( ! $wp_filesystem ) {
		return;
	}

	// Fetch the slider import zip file.
	$response = wp_remote_get( $file_url, array( 'timeout' => 20 ) );
	if ( is_wp_error( $response ) || 200 !== $response['response']['code'] ) {
		return;
	}

	// Save the fetched file into a temporary location.
	$upload_dir = wp_upload_dir();
	$upload_path = trailingslashit( $upload_dir['path'] );
	$timestamp = gmdate( 'Y-m-d__H-i-s' );
	$file_path = $upload_path . 'demo-revslider-import-file_' . $timestamp . '.zip';
	$file_content = wp_remote_retrieve_body( $response );
	if ( ! $wp_filesystem->put_contents( $file_path, $file_content ) ) {
		return;
	}

	// Run the Revolution Slider importer with the fetched file.
	$revslider = new RevSlider;
	$revslider->importSliderFromPost( false, false, $file_path );

	// Delete the temporary file.
	wp_delete_file( $file_path );
}

function phone_repair_init_wp_filesystem() {
	if ( ! function_exists( 'WP_Filesystem' ) ) {
		require_once ABSPATH . 'wp-admin/includes/file.php';
	}

	global $wp_filesystem;
	if ( ! $wp_filesystem ) {
		WP_Filesystem();
	}
	return $wp_filesystem;
}

function phone_repair_recursive_replace( $from, $to, $data ) {
	if ( is_string( $data ) ) {
		$data = str_replace( $from, $to, $data );
	} elseif ( is_array( $data ) ) {
		$_tmp = array();
		foreach ( $data as $key => $value ) {
			$_tmp[ $key ] = phone_repair_recursive_replace( $from, $to, $value );
		}
		$data = $_tmp;
		unset( $_tmp );
	} elseif ( is_object( $data ) ) {
		$_tmp = $data;
		$props = get_object_vars( $data );
		foreach ( $props as $key => $value ) {
			$_tmp->$key = phone_repair_recursive_replace( $from, $to, $value );
		}
		$data = $_tmp;
		unset( $_tmp );
	}
	return $data;
}
